package main

import (
	"context"
	"log"
	"os"
	"time"

	"httpproxycontrol/logging"
	"httpproxycontrol/networking"

	"code.encyclopediaofdaniel.com/dlprows/streamdeck-sdk"
	sdcontext "code.encyclopediaofdaniel.com/dlprows/streamdeck-sdk/context"
)

func main() {
	logging.Enable()
	log.Println("Starting plugin")

	params, err := streamdeck.ParseRegistrationParams(os.Args)
	if err != nil {
		log.Fatalf("error parsing registration params: %v", err)
	}

	sd := streamdeck.NewClient(context.Background(), params)
	defer sd.Close()

	setup(sd)

	if err := sd.Run(); err != nil {
		log.Fatalf("error running streamdeck client: %v\n", err)
	}
}

func setup(client *streamdeck.Client) {
	log.Println("Registering actions")
	action := client.Action("com.dlprows.httpproxy.action")

	contexts := make(map[string]struct{})

	action.RegisterHandler(streamdeck.KeyDown, func(ctx context.Context, client *streamdeck.Client, event streamdeck.Event) error {
		log.Println("Key Down")

		toggleVpn()

		client.SetState(ctx, getState())

		return nil
	})

	action.RegisterHandler(streamdeck.WillAppear, func(ctx context.Context, client *streamdeck.Client, event streamdeck.Event) error {
		log.Println("Will Appear")
		contexts[event.Context] = struct{}{}

		client.SetState(ctx, getState())
		return nil
	})

	action.RegisterHandler(streamdeck.WillDisappear, func(ctx context.Context, client *streamdeck.Client, event streamdeck.Event) error {
		log.Println("Will Disappear")
		delete(contexts, event.Context)
		return nil
	})

	//start background thread to keep the display up to date if changed outside the stream deck
	go func() {
		for range time.Tick(time.Second * 10) {

			for ctxStr := range contexts {
				//for each context
				//build a new context that can be used to perform outbound requests
				ctx := context.Background()
				ctx = sdcontext.WithContext(ctx, ctxStr)

				client.SetState(ctx, getState())
			}
		}
	}()
}

func toggleVpn() error {
	/*
		if on, turn off
		if off, turn on
		if broken, turn off
	*/
	activeDevice, err := networking.GetActiveDevice()

	if err != nil {
		return err
	}

	enabled, err := networking.IsProxyEnabled(activeDevice)

	if err != nil {
		return err
	}

	networking.SetProxyServer(activeDevice, "192.168.41.101", "8888")
	networking.SetProxyState(activeDevice, !enabled)

	return nil
}

func getState() int {
	/*
		0 for no network
		1 for proxy off
		2 for proxy on
		3 for busted
	*/
	activeDevice, err := networking.GetActiveDevice()

	if err != nil {
		return 3
	}

	if activeDevice == "" {
		return 0
	}

	proxyEnabled, err := networking.IsProxyEnabled(activeDevice)

	if err != nil {
		return 3
	}
	if proxyEnabled {
		return 2
	}
	return 1
}
